# Example that shows how to analyse antiKT jets using LCIO format
# Run it using https://atlaswww.hep.anl.gov/asc/jas4pp/ 
# author: S.Chekanov (ANL)

from org.lcsim.event import *
from org.lcsim.util import *
from hep.physics.vec import *
from java.util import *
from java.io import *
from org.lcsim.lcio import LCIOReader
from hep.io.sio import SIOReader
from hep.lcio.implementation.sio import SIOLCReader
from hep.lcio.implementation.io import LCFactory
from hep.lcio.event import *
from hep.lcio.io import *
from jhplot import *  # import graphics
from hephysics.particle import LParticle 
from math import *
from hephysics.particle import LParticle
from hep.physics.jet import FixNumberOfJetsFinder
from hephysics.jet import ParticleD
from java.awt import *
import os,sys

if (len(sys.argv) != 2):
       print "Please specify input directory: jetmass.py input "
       sys.exit()
indir=sys.argv[1]

# http://jwork.org/dmelt/api/doc.php/hephysics/jet/JetN2
from hephysics.jet import JetN2
ktjet=JetN2(0.6,"antikt",1000)     # initialize jets with R=0.6, E-mode, anti-KT,min pT=1000
# print ktjet.info()               # print its settings

# make list of files..
import glob
files=glob.glob(indir+"/*.slcio")
factory = LCFactory.getInstance()
reader = factory.createLCReader()
reader.open(files)

mi=0
ma=500
mbins=100
h1=H1D("Truth",mbins,mi,ma)
h2=H1D("PFO",mbins,mi,ma)
h3=H1D("Clusters",mbins,mi,ma)

nEvent=0
while(1):
     evt=reader.readNextEvent()
     if (evt == None): break
     nEvent=nEvent+1
     # print " file event: ",evt.getEventNumber(), " run=",evt.getRunNumber()
     if (nEvent%100==0): print "# Event: ",nEvent
     strVec = evt.getCollectionNames()
     if nEvent == 1:
            for col in  strVec:
                           print col           

     # if (nEvent>100): break

     # truth-level particles
     # task: take only particles in the central region
     col = evt.getCollection("MCParticle")
     nMc=col.getNumberOfElements()
     particles=ArrayList() # list of particles  
     for i in range(nMc): # loop over all particles 
       par=col.getElementAt(i)
       if(par.getGeneratorStatus() == 1):
              vertex = par.getVertex();
              pdg=par.getPDG()
              momentum = par.getMomentum()
              ee=par.getEnergy()
              mass=par.getMass()
              px=momentum[0]
              py=momentum[1]
              pz=momentum[2]
              charge=par.getCharge()
              # http://jwork.org/dmelt/api/doc.php/hephysics/jet/ParticleD.html
              p=ParticleD(px,py,pz,ee);
              particles.add(p) # add particle to the list 
     ktjet.buildJets(particles)
     Tjets=ktjet.getJetsSorted()      # get a list with sorted jets
     if (len(Tjets)>0):
                 h1.fill(Tjets[0].m())

# See API: http://www.lcsim.org/sites/lcsim/apidocs/org/lcsim/event/
     col = evt.getCollection("PandoraPFOCollection")
     n = col.getNumberOfElements()
     pPFO=ArrayList() # list of particles
     for i in range(n):
          pa=col.getElementAt(i)
          charge=pa.getCharge()
          p4=pa.getMomentum()  
          ee=pa.getEnergy() 
          typep=pa.getType() # type
          p=ParticleD(p4[0],p4[1],p4[2],ee)
          pPFO.add(p)     
     ktjet.buildJets(pPFO)
     Pjets=ktjet.getJetsSorted()      # get a list with sorted jets
     if (len(Pjets)>0):
                 h2.fill(Pjets[0].m())


     col = evt.getCollection("ReconClusters")
     n = col.getNumberOfElements()
     pcluster=ArrayList()
     for i in range(n):
          pa=col.getElementAt(i)
          p4=pa.getPosition()
          ee=pa.getEnergy()
          tmp = sqrt(p4[0]*p4[0] + p4[1]*p4[1] + p4[2]*p4[2])
          px =  ee*p4[0]/tmp
          py =  ee*p4[1]/tmp
          pz =  ee*p4[2]/tmp
          typep=pa.getType() # type
          p=ParticleD(px,py,pz,ee)
          pcluster.add(p)

     ktjet.buildJets(pcluster)
     Cjets=ktjet.getJetsSorted()      # get a list with sorted jets
     if (len(Cjets)>0):
                 h3.fill(Cjets[0].m())

     if (nEvent%10==0 and len(Cjets)>0 and len(Pjets)>0): 
               print nEvent,"pT(lead) truth=%.2f PFO=%.2f Clusters=%.2f [GeV]"%(Tjets[0].perp(),Pjets[0].perp(),Cjets[0].perp()) 

reader.close()


c1=HPlot()
c1.visible()
c1.setAutoRange()
c1.setMarginLeft(100)
c1.setNameX("Jet mass [GeV]")
c1.setNameY("Events")
c1.setRangeX(0,300)
c1.setRangeY(0,0.8)

h1.scale(1.0/h1.entries())
h1.setLineStyle(1)
h1.setPenWidth(3)
h1.setColor(Color.red)
h1.setErrColor(Color.red)

h2.scale(1.0/h2.entries())
h2.setLineStyle(0)
h2.setPenWidth(3)

h3.scale(1.0/h3.entries())
h3.setLineStyle(0)
h3.setPenWidth(3)
h3.setColor(Color.blue)
h3.setErrColor(Color.blue)

c1.draw([h1,h2,h3])

if not os.path.exists("figs"): # make figure 
    os.makedirs("figs")
c1.export("figs/"+indir+".pdf")
print "Created figs/"+indir+".pdf"
